﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Signal_Process;                                   // FFTに必要
using System.IO;                                        // ファイル操作に必要
using WAVE_file;                                        // waveファイルの操作に必要

namespace 鳥の声で種識別
{
    public partial class FFTtestForm : Form
    {
        // グローバル変数を宣言
        wave wave_file = new wave();
        
        
        public FFTtestForm()
        {
            InitializeComponent();
        }

        /*------------------------------------------
         * FFT実行ボタンが押された時の動作
         * 
         * --------------------------------------------*/
        private void FFT_button_Click(object sender, EventArgs e)
        {
            if (this.FFTtimer.Enabled == true) {
                this.FFTtimer.Enabled = false;
                this.TimerStatusLabel.Text = "タイマー停止中";
            }else if (this.FFTtimer.Enabled == false){
                this.FFTtimer.Enabled = true;
                this.TimerStatusLabel.Text = "タイマー起動中";
            }
        }

        /*------------------------------------------
         * waveファイル選択ボタンが押された時の動作
         * 
         * --------------------------------------------*/
        private void wave_set_button_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK) {
                this.wave_file.Open(openFileDialog1);
            }
        }

        private void FFTtimer_Tick(object sender, EventArgs e)
        {
            Graphics g = pictureBox1.CreateGraphics();                      // 図の描画の準備
            Pen blackpen = new Pen(Color.Black, 1);
            Pen redpen = new Pen(Color.Red, 1);
            Pen bluepen = new Pen(Color.Blue, 1);
            int x1, x2, y1, y2;
            double max;

            FFT a = new FFT();                                              // FFTの準備
            const double read_size_sec = 0.05;                              // waveファイルからの読み込み量[s]を指定
            int datasize = (int)FFT.FFT_point.size4096;
            double[] temp;
            int k, i;
            wave.MusicDatas[] data;                                         // 音声ファイルから読み込んだデータ列が格納される。


            if (this.wave_file.IsOpen && this.wave_file.ReadLimit == false) // 読めるなら、読み込む
            {
                // read
                data = this.wave_file.ReadSec(read_size_sec);               // クラスメソッドを使用してファイルからread_size_sec秒間読み込み。実行の度に更新される。
                temp = new double[this.wave_file.GetArrSize(read_size_sec)];// FFT用に値を格納しなおす配列のインスタンスを生成
                for (i = 0; i < temp.Length; i++)
                {                        // データをコピー
                    temp[i] = (double)data[i].Right;
                }

                // view
                pictureBox1.Refresh();
                g.DrawLine(blackpen, 0, pictureBox1.Height / 2, pictureBox1.Width, pictureBox1.Height / 2); // 軸を書く
                max = 0;                                                    // 振幅の最大値を探す
                for (i = 0; i < temp.Length; i++)
                {
                    if (temp[i] > max) max = temp[i];
                }
                x1 = y1 = x2 = y2 = 0;
                for (i = 0; i < temp.Length - 1; i++)                       // 読み込んだデータを描画。最大値で正規化。
                {
                    x1 = x2;
                    y1 = y2;
                    x2 = (i + 1) * pictureBox1.Width / temp.Length;
                    y2 = (int)(-(pictureBox1.Height / 2) * temp[i + 1] / max + pictureBox1.Height / 2);
                    g.DrawLine(blackpen, x1, y1, x2, y2);
                }

                // FFT
                a.FFT_calc(temp, datasize);                                 // FFTを実行

                // view
                max = 0;                                                    // 最大値を探す
                for (k = 0; k < datasize; k++)
                {
                    if (a.F_re[k] > max) max = a.F_re[k];
                    if (a.F_im[k] > max) max = a.F_im[k];
                }
                x1 = y1 = x2 = y2 = 0;
                for (i = 0; i < a.F_re.Length - 1; i++)                     // リアルパートのみ表示
                {
                    x1 = x2;
                    y1 = y2;
                    x2 = (i + 1) * pictureBox1.Width / datasize;            // FFT実部を描画。最大値で正規化。
                    y2 = (int)(-(pictureBox1.Height / 2) * a.F_re[i + 1] / max + pictureBox1.Height / 2);
                    g.DrawLine(redpen, x1, y1, x2, y2);
                }
                this.TimeAtReadLabel.Text = "読み出し時刻 = " + this.wave_file.NowTime + "[s]";
            }
            g.Dispose();
        }

    }
}
