﻿/**************************************
 * waveファイルを読み込んで、順次FFTを実行・描画するサンプルプログラム
 * [開発者]     森下功啓(Katsuhiro Morishita)
 * 
 * [参考文献]   オブジェクト配列について，http://www.apfield.jp/csgarden/checkbox.html
 * 
 * [メモ]
 *  本ソースコードを利用してFFTについて勉強される方には、デバッグモードで一行ずつ実行したりブレークポイントをあちこちに作って動作を確認することをお勧めします。
 *  おまけとしてオブジェクトを配列で宣言して制御するコードのテストを兼ねています。
 *  このコードを見本とすれば、テキストボックスの動的配置は容易いはずです。
 *  なお、音声を同時に流そうとしてWindowsMediaPlayerのDLLを使用するように変更したのですが、おかげでx64 CPU（64ビットCPU）では動作しなくなりました。どーしよう？
 *  
 * [履歴]       2011/3      開発を始めたのは確かこの時期だったと思う
 *              2011/10/7   waveクラスの更新に合わせてコードを更新
 *              2011/11/15  FFTクラスの更新に合わせてコードを更新
 *                          グラフの描画を高速化して、リアルタイム処理が可能になった。
 *                          音声検出機能も追加
 *                          FFTに合わせて、リアルタイムで音を再生する機能を追加（この機能は今後、削除する可能性が高い）
 * *********************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Signal_Process;                                   // FFTに必要
using System.IO;                                        // ファイル操作に必要
using WAVE_file;                                        // waveファイルの操作に必要

namespace 鳥の声で種識別
{
    public partial class FFTtestForm : Form
    {
        // グローバル変数を宣言
        private const int FFT_arr_size = 5;
        private wave wave_file = new wave();
        private CheckBox[] fft_chbox = new CheckBox[FFT_arr_size];                      // チェックボックスオブジェクトを配列で宣言
        private int[] fft_points = new int[FFT_arr_size] { 256, 512, 1024, 2048, 4096 };// 実施するFFTのサイズを指定するのに使用する
        private SignalBasic sensor;                                                     // 鳴き声検出のための検査クラス
        private DateTime timeAtStart4Player;                                            // 再生開始時刻をメモる
        private TimeSpan playedTime = new TimeSpan(0);                                  // トータルの再生時間をメモる

        /// <summary>
        /// コンストラクタ
        /// 始めに実行される
        /// </summary>
        public FFTtestForm()
        {
            InitializeComponent();
        }
        /// <summary>
        /// デスコンストラクタ
        /// </summary>
        ~FFTtestForm()
        {

        }
        /// <summary>
        /// タイマーを動かしたり、停止させる
        /// </summary>
        private void SwitchPlay()
        {
            if (this.FFTtimer.Enabled == true)
            {
                this.FFTtimer.Enabled = false;
                this.TimerStatusLabel.Text = "タイマー停止中";
                // 再生していれば止める
                this.axWindowsMediaPlayer1.Ctlcontrols.pause();
                this.playedTime = (DateTime.Now - this.timeAtStart4Player).Add(this.playedTime);// 再生時間を計算する
            }
            else if (this.FFTtimer.Enabled == false)
            {
                this.FFTtimer.Enabled = true;
                this.TimerStatusLabel.Text = "タイマー起動中";
                // 再生する
                this.axWindowsMediaPlayer1.Ctlcontrols.play();
                this.timeAtStart4Player = DateTime.Now;                             // 再生開始時刻をメモる
            }
            return;
        }
        /// <summary>
        /// FFT実行ボタンが押された時の動作
        /// タイマーを動かしたり、停止させる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FFT_button_Click(object sender, EventArgs e)
        {
            this.SwitchPlay();
            return;
        }
        /// <summary>
        /// waveファイル選択ボタンが押された時の動作
        /// ファイルを選択し、可能なら開く
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void wave_set_button_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = "hoge.wave";                                 // ファイルを開く前に初期設定
            openFileDialog1.Filter = "Wave Files (*.wav, *.wave)|*.wav;*.wave";
            if (openFileDialog1.ShowDialog() == DialogResult.OK) {
                this.wave_file.Open(openFileDialog1);
            }
            // 音の再生関係
            this.axWindowsMediaPlayer1.Ctlcontrols.stop();                          // 再生を停止
            this.axWindowsMediaPlayer1.URL = openFileDialog1.FileName;              // ファイル名をセット
            if (this.FFTtimer.Enabled == true) { this.axWindowsMediaPlayer1.Ctlcontrols.play(); this.timeAtStart4Player = DateTime.Now; }// タイマーが動作中なら即座に再生を開始
            this.playedTime = new TimeSpan(0);                                      // トータルの再生時間初期化
            // 鳴き声検出関係
            // キビタキ     2500-4500
            // ヨタカ       650-1750
            // アオバズク   550-750
            // ジュウイチ   3200-5000
            this.sensor = new SignalBasic(1, 4500, 2500, new TimeSpan(0, 0, 3), new TimeSpan(0, 0, 0, 0, 100));   // 鳴き声検出のための検査クラスを初期化（初期化しないと、以前のファイルの影響を引きずる）
        }
        /// <summary>
        /// タイマーで定期的に実行される
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FFTtimer_Tick(object sender, EventArgs e)
        {
            int x, y;
            const double read_size_sec = 0.1;                                           // waveファイルからの読み込み量[s]を指定
            wave.MusicData data;                                                        // 音声ファイルから読み込んだデータが格納される。

            if (this.wave_file.IsOpen && this.wave_file.ReadLimit == false)             // 読めるなら、読み込む
            {
                // メディアプレーヤの再生とFFTの同期を図る
                TimeSpan ts = (DateTime.Now - this.timeAtStart4Player).Add(this.playedTime);
                if (ts.TotalSeconds > this.wave_file.NowTime)                           // 再生時間と読み出し時間を常に比較して追いつかれたらFFTを実施する。なので少しだけディレイがあるけど肉眼ではほとんどわからないよね？
                {
                    // read
                    data = this.wave_file.Read(read_size_sec);                          // クラスメソッドを使用してファイルからread_size_sec秒間読み込み。実行の度に更新される。
                    double[] sound;
                    if (data.UsableCH == wave.MusicData.UsableChannel.Left)             // データの入っている方を格納する
                        sound = data.GetData(wave.MusicData.Channel.Left);              // 
                    else
                        sound = data.GetData(wave.MusicData.Channel.Right);

                    // wave data plot
                    pictureBox1.Refresh();
                    Pen wave_kpen = new Pen(Color.Black, 1);
                    Graphics g = pictureBox1.CreateGraphics();                          // 図の描画の準備
                    g.DrawLine(wave_kpen, 0, pictureBox1.Height / 2, pictureBox1.Width, pictureBox1.Height / 2); // 軸を書く
                    Point[] wavePoint = new Point[500];
                    int l = 0;
                    int step = sound.Length / wavePoint.Length;
                    for (int i = 0; i < wavePoint.Length; i++, l++)                     // 読み込んだデータを描画
                    {
                        x = i * pictureBox1.Width / wavePoint.Length;
                        y = (int)(-(pictureBox1.Height / 2) * sound[i * step] / (2 << this.wave_file.Resolution - 1) + pictureBox1.Height / 2);
                        wavePoint[l] = new Point(x, y);
                    }
                    g.DrawLines(wave_kpen, wavePoint);
                    wave_kpen.Dispose();

                    // FFT
                    for (int i = 0; i < FFT_arr_size; i++)
                    {
                        // plot FFT result
                        if (this.fft_chbox[i].Checked)
                        {
                            // FFTの実施
                            DFT[] fft_arr = new DFT[FFT_arr_size];                                                  // FFTの準備
                            fft_arr[i] = new DFT(fft_points[i], DFT.Window.Perzen);                                 // インスタンスを生成
                            FFTresult result = fft_arr[i].FFT(sound, (long)this.wave_file.SamplingRate);            // FFTを実行. FFTサイズが音声データよりも小さい場合、始めの方しか処理しないので注意.
                            string cd = Directory.GetCurrentDirectory();                                            // カレントディレクトリ（作業フォルダ）を取得
                            this.Label_savingFolder.Text = "Data loging to:" + cd;                                  // ファイルの保存先を表示
                            result.Save(cd + @"\FFT result" + fft_arr[i].FFTsize.ToString() + ".csv");              // 結果をファイルに保存
                            // 音声検出
                            if (this.fft_chbox[4].Checked && i == 4) this.sensor.Set(result);                       // 5番のFFT結果のみ使用して音声検出
                            this.Label_detection.Text = "Detection: " + this.sensor.Detection.ToString();           // 結果をUIのラベルに出力
                            if (this.sensor.Detection)                                                              // ラベルの色も変える
                                this.Label_detection.BackColor = Color.LawnGreen;
                            else
                                this.Label_detection.BackColor = Color.Red;
                            this.sensor.Save(0, cd + @"\Detection log.csv");                                         // ファイルへ状況を出力（主にデバッグ用）
                            // FFTの結果を描画
                            Point[] fftPoint = new Point[1000];
                            for (int k = 0; k < fftPoint.Length; k++)                                               // 描画スピードを優先
                            {
                                x = k * pictureBox1.Width / fftPoint.Length;                                        // 画面の幅にスケーリング
                                y = (int)(-(pictureBox1.Height / 2) * result.GetData(k * result.Length / fftPoint.Length).data.Log / 5 + pictureBox1.Height);
                                if (y > pictureBox1.Height)
                                    y = pictureBox1.Height;
                                else if (y < 0)
                                    y = 0;
                                fftPoint[k] = new Point(x, y);
                            }
                            Pen fft_pen = new Pen(Color.FromArgb(40 * i, 250 - 50 * i, i * 50), 1);                 // カラーの決定
                            g.DrawLines(fft_pen, fftPoint);
                            fft_pen.Dispose();
                        }
                    }
                    g.Dispose();
                    this.TimeAtReadLabel.Text = "読み出し時刻 = " + this.wave_file.NowTime.ToString("0000.0") + "[s]";
                }
            }
            else if (this.wave_file.ReadLimit) { this.TimeAtReadLabel.Text = "読み込みは終了しました。"; this.SwitchPlay(); }
            else { this.TimeAtReadLabel.Text = "ファイルの読み出しに失敗しました。フォーマットを確認してください。"; }
            
        }
        /// <summary>
        /// フォームがロードされた際に、最初に実行する内容
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FFTtestForm_Load(object sender, EventArgs e)
        {
             // チェックボックスの設定
            for (int i = 0; i < FFT_arr_size; i++)
            {
                this.fft_chbox[i] = new CheckBox();                         // チェックボックス生成
                this.fft_chbox[i].Location = new Point( 10, 20 * i);        // フォーム上に20ピクセル間隔で配置
                this.fft_chbox[i].Width = 80;
                Controls.Add(this.fft_chbox[i]);                            // チェックボックスをコントロールとして追加
                this.fft_chbox[i].Text = "FFT" + this.fft_points[i].ToString();
                this.fft_chbox[i].Tag = Convert.ToString(i);　              // チェックボックス判別用のタグを付ける
                this.fft_chbox[i].BringToFront();                           // 最前面に表示
            }
            // メディアプレーヤーの設定
            // URLプロパティが指定されても自動的に再生されないようにする
            this.axWindowsMediaPlayer1.settings.autoStart = false;
        }
    }
}
